// Copyright (C) 2017 Ben Asselstine
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
#ifndef XGES_CREATE_CLIP_H
#define XGES_CREATE_CLIP_H 1
#include <argp.h>
#include "xges-create.h"
#include "promote.h"
typedef void* clip_t;
void init_clip (clip_t *clip, int layer);
void init_clip_from_clip (clip_t *clip, clip_t clip_to_copy_from);
void destroy_clip (clip_t clip);
extern struct argp clip_argp;
int is_a_clip_option (int key);
void clip_validate (clip_t clip, struct argp_state *state);
int clip_has_transition (clip_t clip);
char *clip_get_uri (clip_t clip);
int clip_get_width (clip_t clip);
int clip_get_height (clip_t clip);
int clip_has_crop (clip_t clip);
int clip_crop_extents_in_bounds (clip_t clip);
int clip_has_fade (clip_t clip);
int clip_fade_in_bounds (clip_t clip);
void clip_set_file (clip_t clip, char *file);
void clip_fill_zoom (clip_t clip);
int clip_has_zoom (clip_t clip);
int clip_zoom_point_in_bounds (clip_t clip);
int clip_has_splice (clip_t clip);
void insert_clip_after (clip_t clip, int idx, clip_t **clips, int *num_clips);
int clip_splice (clip_t c, clip_t **clips, int *num_clips);
void clip_set_duration (clip_t clip, double duration);
double clip_get_duration (clip_t clip);
unsigned long long int clip_get_cached_duration (clip_t clip);
double clip_get_start (clip_t clip);
void clip_set_start (clip_t clip, double start);
void clip_set_fade_in (clip_t clip, double fade);
void clip_set_fade_out (clip_t clip, double fade);
int clip_promote (promote_t p, clip_t **clips, int *num_clips, int num_layers);
int clip_get_layer_number (clip_t clip);
double clip_get_fade_in (clip_t clip);
double clip_get_fade_out (clip_t clip);
int clip_get_crop_width (clip_t clip);
int clip_get_crop_height (clip_t clip);
int clip_get_crop_x (clip_t clip);
int clip_get_crop_y (clip_t clip);
int clip_get_zoom_x (clip_t clip);
int clip_get_zoom_y (clip_t clip);
double clip_get_zoom_percent (clip_t clip);
int clip_get_greenscreen (clip_t clip);
char *clip_get_transition_name (clip_t clip);
double clip_get_transition_duration (clip_t clip);
void clip_set_position (clip_t clip, double pos);
double clip_get_position (clip_t clip);
double clip_get_relative_position (clip_t clip);
void clip_set_layer (clip_t clip, int layer);
void clip_clear_fade (clip_t clip);
void clip_clear_transition (clip_t clip);
void clip_remove_video_track (clip_t clip);
void clip_remove_audio_track (clip_t clip);
int clip_has_audio_track (clip_t clip);
int clip_has_video_track (clip_t clip);
int clip_is_audio_only (clip_t clip);
int clip_has_position (clip_t clip);
int clip_has_relative_position (clip_t clip);
void clip_clear_position (clip_t clip);
#endif
