// Copyright (C) 2017 Ben Asselstine
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <argp.h>
#include <stdlib.h>
#include "layer-priv.h"
#include "xges-create-priv.h"

void
init_layer (layer_t *layer, int id)
{
  size_t length = sizeof (struct layer_details_t);
  *layer = calloc (1, length);
  struct layer_details_t *p = *layer;
  p->id = id;
  return;
}

void
destroy_layer (layer_t layer)
{
  free (layer);
  return;
}

static error_t
parse_layer_opt (int key, char *arg, struct argp_state *state)
{
  struct xges_create_details_t *xges_create = (struct xges_create_details_t*) state->input;
  char *end;
  if (xges_create->num_layers == 1 && !is_a_layer_option (key))
    return 0;
  else if (xges_create->num_layers == 1 && is_a_layer_option (key))
    argp_error (state, "you must specify `" LAYER_CMD "' before using one of its options");
  else
    check_bad_option (key, state);
  struct layer_details_t *l = (struct layer_details_t*) xges_create->layers[xges_create->num_layers-1];
  switch (key)
    {
    case LAYER_MUTE:
      l->mute = 1;
      break;
    }
  return 0;
}

static struct argp_option 
layer_options[] = 
{
    { 0, 0, 0, 0, "Layer Options:", LAYER_OPTION },
    { "mute", LAYER_MUTE, 0, 0,
      "Set the volume of all clips on this layer to zero", LAYER_OPTION },
    { 0 }
};

int
is_a_layer_option (int key)
{
  switch (key)
    {
    case LAYER_MUTE:
      return 1;
    }
  return 0;
}

struct argp
layer_argp =
{
  layer_options, parse_layer_opt, 0, 0, 0
};
