// Copyright (C) 2017 Ben Asselstine
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <argp.h>
#include <stdlib.h>
#include "promote-priv.h"
#include "xges-create-priv.h"

void
init_promote (promote_t *promote)
{
  size_t length = sizeof (struct promote_details_t);
  *promote = calloc (1, length);
  struct promote_details_t *p = *promote;
  return;
}

void
destroy_promote (promote_t promote)
{
  free (promote);
  return;
}

int
promote_get_layer (promote_t promote)
{
  struct promote_details_t *p = promote;
  return p->layer;
}

double
promote_get_position (promote_t promote)
{
  struct promote_details_t *p = promote;
  return p->position;
}

double
promote_get_duration (promote_t promote)
{
  struct promote_details_t *p = promote;
  return p->duration;
}

void
promote_set_layer (promote_t promote, int layer)
{
  struct promote_details_t *p = promote;
  p->layer = layer;
}

void
promote_set_position (promote_t promote, double position)
{
  struct promote_details_t *p = promote;
  p->position = position;
}

void
promote_set_duration (promote_t promote, double duration)
{
  struct promote_details_t *p = promote;
  p->duration = duration;
}

int
promote_get_justmakehole (promote_t promote)
{
  struct promote_details_t *p = promote;
  return p->justmakehole;
}

int
promote_get_destination_layer (promote_t promote)
{
  struct promote_details_t *p = promote;
  return p->destination_layer;
}

static error_t
parse_promote_opt (int key, char *arg, struct argp_state *state)
{
  struct xges_create_details_t *xges_create = (struct xges_create_details_t*) state->input;
  char *end;
  check_bad_option (key, state);
  struct promote_details_t *p = NULL;
  if (xges_create->num_promotions)
    p = (struct promote_details_t*) xges_create->promotions[xges_create->num_promotions-1];
  switch (key)
    {
    case PROMOTE_DEST_LAYER:
        {
          unsigned long int val = strtoul (arg, &end, 0);
          if (*end != '\0')
            argp_error (state, "`%s' is a bad value for --destination", arg);
          else
            p->destination_layer = val;
        }
      break;
    case PROMOTE_JUSTMAKEHOLE:
      p->justmakehole = 1;
      break;
    }
  return 0;
}

static struct argp_option 
promote_options[] = 
{
    { 0, 0, 0, 0, "Promote Options:", PROMOTE_OPTION },
    { "destination", PROMOTE_DEST_LAYER, "NUM", 0,
      "Move promoted region to layer NUM instead of zero", PROMOTE_OPTION },
    { "just-make-hole", PROMOTE_JUSTMAKEHOLE, 0, OPTION_HIDDEN,
      "only make cuts on layers prior to LAYER", PROMOTE_OPTION },
    { 0 }
};

int
is_a_promote_option (int key)
{
  switch (key)
    {
    case PROMOTE_DEST_LAYER:
    case PROMOTE_JUSTMAKEHOLE:
      return 1;
    }
  return 0;
}

struct argp
promote_argp =
{
  promote_options, parse_promote_opt, 0, 0, 0
};
