// Copyright (C) 2017 Ben Asselstine
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#define _GNU_SOURCE
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <argp.h>
#include <argz.h>
#include <stdlib.h>
#include "xges-create-priv.h"
#include "clip.h"
#include "layer.h"
#include "promote.h"
#include "generate.h"

int next_arg_is_file;
int next_arg_is_promote_layer;
int next_arg_is_promote_position;
int next_arg_is_promote_duration;
char *transitions[] = {
 "bar-wipe-lr",
 "bar-wipe-tb",
 "box-wipe-tl",
 "box-wipe-tr",
 "box-wipe-br",
 "box-wipe-bl",
 "four-box-wipe-ci",
 "four-box-wipe-co",
 "barndoor-v",
 "barndoor-h",
 "box-wipe-tc",
 "box-wipe-rc",
 "box-wipe-bc",
 "box-wipe-lc",
 "diagonal-tl",
 "diagonal-tr",
 "bowtie-v",
 "bowtie-h",
 "barndoor-dbl",
 "barndoor-dtl",
 "misc-diagonal-dbd",
 "misc-diagonal-dd",
 "vee-d",
 "vee-l",
 "vee-u",
 "vee-r",
 "barnvee-d",
 "barnvee-l",
 "barnvee-u",
 "barnvee-r",
 "iris-rect",
 "clock-cw1",
 "clock-cw3",
 "clock-cw6",
 "clock-cw9",
 "pinwheel-tbv",
 "pinwheel-tbh",
 "pinwheel-fb",
 "fan-ct",
 "fan-cr",
 "doublefan-fov",
 "doublefan-foh",
 "singlesweep-cwt",
 "singlesweep-cwr",
 "singlesweep-cwb",
 "singlesweep-cwl",
 "doublesweep-pv",
 "doublesweep-pd",
 "doublesweep-ov",
 "doublesweep-oh",
 "fan-t",
 "fan-r",
 "fan-b",
 "fan-l",
 "doublefan-fiv",
 "doublefan-fih",
 "singlesweep-cwtl",
 "singlesweep-cwbl",
 "singlesweep-cwbr",
 "singlesweep-cwtr",
 "doublesweep-pdtl",
 "doublesweep-pdbl",
 "saloondoor-t",
 "saloondoor-l",
 "saloondoor-b",
 "saloondoor-r",
 "windshield-r",
 "windshield-u",
 "windshield-v",
 "windshield-h",
 "crossfade",
   0
};

char *transition_desc[] = {
  "A bar moves from left to right",
  "A bar moves from top to bottom",
  "A box expands from the upper-left corner to the lower-right corner",
  "A box expands from the upper-right corner to the lower-left corner",
  "A box expands from the lower-right corner to the upper-left corner",
  "A box expands from the lower-left corner to the upper-right corner",
  "A box shape expands from each of the four corners toward the center",
  "A box shape expands from the center of each quadrant toward the corners of each quadrant",
  "A central, vertical line splits and expands toward the left and right edges",
  "A central, horizontal line splits and expands toward the top and bottom edges",
  "A box expands from the top edge's midpoint to the bottom corners",
  "A box expands from the right edge's midpoint to the left corners",
  "A box expands from the bottom edge's midpoint to the top corners",
  "A box expands from the left edge's midpoint to the right corners",
  "A diagonal line moves from the upper-left corner to the lower-right corner",
  "A diagonal line moves from the upper right corner to the lower-left corner",
  "Two wedge shapes slide in from the top and bottom edges toward the center",
  "Two wedge shapes slide in from the left and right edges toward the center",
  "A diagonal line from the lower-left to upper-right corners splits and expands toward the opposite corners",
  "A diagonal line from upper-left to lower-right corners splits and expands toward the opposite corners",
  "Four wedge shapes split from the center and retract toward the four edges",
  "A diamond connecting the four edge midpoints simultaneously contracts toward the center and expands toward the edges",
  "A wedge shape moves from top to bottom",
  "A wedge shape moves from right to left",
  "A wedge shape moves from bottom to top",
  "A wedge shape moves from left to right",
  "A 'V' shape extending from the bottom edge's midpoint to the opposite corners contracts toward the center and expands toward the edges",
  "A 'V' shape extending from the left edge's midpoint to the opposite corners contracts toward the center and expands toward the edges",
  "A 'V' shape extending from the top edge's midpoint to the opposite corners contracts toward the center and expands toward the edges",
  "A 'V' shape extending from the right edge's midpoint to the opposite corners contracts toward the center and expands toward the edges",
  "A rectangle expands from the center",
  "A radial hand sweeps clockwise from the twelve o'clock position",
  "A radial hand sweeps clockwise from the three o'clock position",
  "A radial hand sweeps clockwise from the six o'clock position",
  "A radial hand sweeps clockwise from the nine o'clock position",
  "Two radial hands sweep clockwise from the twelve and six o'clock positions",
  "Two radial hands sweep clockwise from the nine and three o'clock positions",
  "Four radial hands sweep clockwise",
  "A fan unfolds from the top edge, the fan axis at the center",
  "A fan unfolds from the right edge, the fan axis at the center",
  "Two fans, their axes at the center, unfold from the top and bottom",
  "Two fans, their axes at the center, unfold from the left and right",
  "A radial hand sweeps clockwise from the top edge's midpoint",
  "A radial hand sweeps clockwise from the right edge's midpoint",
  "A radial hand sweeps clockwise from the bottom edge's midpoint",
  "A radial hand sweeps clockwise from the left edge's midpoint",
  "Two radial hands sweep clockwise and counter-clockwise from the top and bottom edges' midpoints",
  "Two radial hands sweep clockwise and counter-clockwise from the left and right edges' midpoints",
  "Two radial hands attached at the top and bottom edges' midpoints sweep from right to left",
  "Two radial hands attached at the left and right edges' midpoints sweep from top to bottom",
  "A fan unfolds from the bottom, the fan axis at the top edge's midpoint",
  "A fan unfolds from the left, the fan axis at the right edge's midpoint",
  "A fan unfolds from the top, the fan axis at the bottom edge's midpoint",
  "A fan unfolds from the right, the fan axis at the left edge's midpoint",
  "Two fans, their axes at the top and bottom, unfold from the center",
  "Two fans, their axes at the left and right, unfold from the center",
  "A radial hand sweeps clockwise from the upper-left corner",
  "A radial hand sweeps counter-clockwise from the lower-left corner",
  "A radial hand sweeps clockwise from the lower-right corner",
  "A radial hand sweeps counter-clockwise from the upper-right corner",
  "Two radial hands attached at the upper-left and lower-right corners sweep down and up",
  "Two radial hands attached at the lower-left and upper-right corners sweep down and up",
  "Two radial hands attached at the upper-left and upper-right corners sweep down",
  "Two radial hands attached at the upper-left and lower-left corners sweep to the right",
  "Two radial hands attached at the lower-left and lower-right corners sweep up",
  "Two radial hands attached at the upper-right and lower-right corners sweep to the left",
  "Two radial hands attached at the midpoints of the top and bottom halves sweep from right to left",
  "Two radial hands attached at the midpoints of the left and right halves sweep from top to bottom",
  "Two sets of radial hands attached at the midpoints of the top and bottom halves sweep from top to bottom and bottom to top",
  "Two sets of radial hands attached at the midpoints of the left and right halves sweep from left to right and right to left",
  "Crossfade",
  0
};

int
is_a_transition (char *s)
{
  char **t;
  t = transitions;
  while (*t != 0)
    {
      if (strcmp (s, *t) == 0)
        return 1;
      t++;
    }
  return 0;
}

void
init_xges_create (xges_create_t *xges_create)
{
  struct xges_create_details_t *t;
  size_t length = sizeof (struct xges_create_details_t);
  *xges_create = calloc (1, length);
  t = (struct xges_create_details_t*) *xges_create;

  t->num_clips = 0;
  t->clips = NULL;
  t->num_layers = 1;
  t->layers = malloc (sizeof (layer_t));
  init_layer (&t->layers[0], 0);
  t->framerate_numerator = 30;
  t->framerate_denominator = 1;
  t->zoom = 100.0;
  t->display_width = 78; //columns
  return;
}

int 
get_xges_create_mode (xges_create_t xges_create)
{
  struct xges_create_details_t *t = (struct xges_create_details_t*) xges_create;
  return t->mode;
}

void
destroy_xges_create (xges_create_t xges_create)
{
  int i;
  struct xges_create_details_t *t = (struct xges_create_details_t*) xges_create;
  for (i = 0; i < t->num_clips; i++)
    destroy_clip (t->clips[i]);
  free (t->clips);
  for (i = 0; i < t->num_layers; i++)
    destroy_layer (t->layers[i]);
  free (t->layers);
  free (xges_create);
  return;
}

static const struct argp_child 
child_parsers[] = 
{
    { &clip_argp },
    { &layer_argp },
    { &promote_argp },
    { 0 },
};

static struct argp_option 
xges_create_options[] = 
{
    { "width", XGESC_WIDTH, "PIXELS", 0,
      "The width of the video project", 3 },
    { "height", XGESC_HEIGHT, "PIXELS", 0,
      "The height of the video project", 2 },
    { "framerate", XGESC_FRAMERATE, "FRACTION", 0,
      "The frames per second as a fraction (default 1/30)", 4 },
    { "frame-rate", XGESC_FRAMERATE, "FRACTION", OPTION_ALIAS | OPTION_HIDDEN },
    { "render", XGESC_RENDER, "FILE", 0,
      "Render the .xges file with ges-launch-1.0 and put the rendered video in FILE (must be .webm)", 1},
    { "timeline", XGESC_DISPLAY, 0, 0,
      "Show a textual representation of the layers instead of making a .xges file", 6},
    { "timeline-width", XGESC_DISPLAY_WIDTH, "COLS", OPTION_HIDDEN,
      "Display the layers in COLS columns instead of 80", 6},
    { "output", XGESC_OUTFILE, "FILE", 0,
      "Write the .xges file to FILE instead of the standard output", 1},
    { "scale-render", XGESC_ZOOM, "PERCENT", OPTION_HIDDEN,
      "Scale the video by PERCENT (default 100)", 5},
    { "list-transitions", XGESC_LIST_TRANSITIONS, 0, 0,
      "Give the available transitions and exit", 7},
    { "dump-clips", XGESC_DUMP_CLIPS, 0, OPTION_HIDDEN,
      "Instead of generating a .xges file, just show the clips.", 8},
    { "preview", XGESC_PREVIEW, 0, 0,
      "Display the project with ges-launch-1.0", 8},
    { 0 }
};

static int
is_an_option (int key, xges_create_t t)
{
  struct xges_create_details_t *xges_create = (struct xges_create_details_t*) t;
  switch (key)
    {
    case ARGP_KEY_ARG:
    case ARGP_KEY_ARGS:
    case ARGP_KEY_END:
    case ARGP_KEY_NO_ARGS:
    case ARGP_KEY_INIT:
    case ARGP_KEY_FINI:
    case ARGP_KEY_SUCCESS:
    case ARGP_KEY_ERROR:
    case 'h':
    case '?':
    case 'V':
      return 1;
    };

  if (xges_create->mode == 1)
    return is_a_clip_option (key);
  else if (xges_create->mode == 2)
    return is_a_layer_option (key);
  else if (xges_create->mode == 3)
    return is_a_promote_option (key);
  return 0;
}

void
check_bad_option (int key, struct argp_state *state)
{
  struct xges_create_details_t *t = (struct xges_create_details_t*) state->input;
  if (!is_an_option (key, t))
    {
      if (t->mode == 1)
        argp_error (state, "bad option passed to " CLIP_CMD);
      else if (t->mode == 2)
        argp_error (state, "bad option passed to " LAYER_CMD);
      else if (t->mode == 3)
        argp_error (state, "bad option passed to " PROMOTE_CMD);
      else 
        argp_error (state, "bad option passed!");
    }
}

static void
remember_commandline (struct xges_create_details_t *t, char **argv)
{
  char *argz = NULL;
  size_t argz_len = 0;
  argz_create (&argv[1], &argz, &argz_len);
  argz_stringify (argz, argz_len, ' ');
  t->commandline = argz;
}

static error_t
parse_xges_create_opt (int key, char *arg, struct argp_state *state)
{
  char *end;
  const struct argp_option *opt;
  struct xges_create_details_t *xges_create = (struct xges_create_details_t*) state->input;
  switch (key)
    {
    case XGESC_FRAMERATE:
        {
          unsigned int a, b;
          int retval = sscanf (arg, "%u/%u", &a, &b);
          if (retval != 2)
            argp_error (state, "`%s' is a bad value for --framerate", arg);
          else
            {
              xges_create->framerate_numerator = a;
              xges_create->framerate_numerator = b;
            }
        }
      break;
    case XGESC_WIDTH:
        {
          unsigned long val = strtoul (arg, &end, 0);
          if (*end != '\0')
            argp_error (state, "`%s' is a bad value for --width", arg);
          else
            xges_create->width = val;
        }
      break;
    case XGESC_HEIGHT:
        {
          unsigned long val = strtoul (arg, &end, 0);
          if (*end != '\0')
            argp_error (state, "`%s' is a bad value for --height", arg);
          else
            xges_create->height = val;
        }
      break;
    case XGESC_DISPLAY:
      xges_create->display = 1;
      break;
    case XGESC_DISPLAY_WIDTH:
        {
          unsigned long val = strtoul (arg, &end, 0);
          if (*end != '\0' || val < 10)
            argp_error (state, "`%s' is a bad value for --display-width", arg);
          else
            xges_create->display_width = val;
        }
      break;
    case XGESC_RENDER:
      xges_create->render_uri = arg;
      break;
    case XGESC_LIST_TRANSITIONS:
      xges_create->list_transitions = 1;
      break;
    case XGESC_OUTFILE:
        {
          FILE *fp = fopen (arg, "w");
          if (!fp)
            argp_error (state, "cannot open `%s' for writing (%m)", arg);
          else
            {
              xges_create->output_filename = arg;
              fclose (fp);
            }
        }
      break;
    case XGESC_ZOOM:
        {
          double val = strtod (arg, &end);
          if (*end != '\0' || val < 0 || val > 100)
            argp_error (state, "`%s' is a bad value for --zoom", arg);
          else
            xges_create->zoom = val;
        }
      break;
    case XGESC_DUMP_CLIPS:
      xges_create->dump_clips = 1;
      break;
    case XGESC_PREVIEW:
      xges_create->preview = 1;
      break;
    case ARGP_KEY_ARG:
      if (strcmp (arg, CLIP_CMD) == 0 && !next_arg_is_file)
        {
          if (xges_create->num_clips)
            clip_validate (xges_create->clips[xges_create->num_clips-1], state);
          xges_create->mode = 1;
          xges_create->num_clips++;
          xges_create->clips = realloc ((clip_t*) xges_create->clips, sizeof (clip_t) * xges_create->num_clips);
          init_clip (&xges_create->clips[xges_create->num_clips-1], xges_create->num_layers - 1);
          next_arg_is_file = 1;
        }
      else if (strcmp (arg, LAYER_CMD) == 0 && !next_arg_is_file)
        {
          if (xges_create->num_clips)
            clip_validate (xges_create->clips[xges_create->num_clips-1], state);
          xges_create->mode = 2;
          xges_create->num_layers++;
          xges_create->layers = realloc ((layer_t*) xges_create->layers, sizeof (layer_t) * xges_create->num_layers);
          init_layer (&xges_create->layers[xges_create->num_layers-1], xges_create->num_layers-1);
        }
      else if (strcmp (arg, PROMOTE_CMD) == 0 && !next_arg_is_file)
        {
          xges_create->mode = 3;
          next_arg_is_promote_layer = 1;
        }
      else if (next_arg_is_promote_layer)
        {
          next_arg_is_promote_layer = 0;
          unsigned long val = strtoul (arg, &end, 0);
          if (*end != '\0')
            argp_error (state, "`%s' is a bad value for a layer number", arg);
          else
            {
              xges_create->num_promotions++;
              xges_create->promotions = realloc ((layer_t*) xges_create->promotions, sizeof (promote_t) * xges_create->num_promotions);
              init_promote (&xges_create->promotions[xges_create->num_promotions-1]);
              promote_set_layer (xges_create->promotions[xges_create->num_promotions-1], val);
            }
          next_arg_is_promote_position = 1;
        }
      else if (next_arg_is_promote_position)
        {
          next_arg_is_promote_position = 0;
          next_arg_is_promote_duration = 1;
          double val = strtod (arg, &end);
          if (val >= 0)
            promote_set_position (xges_create->promotions[xges_create->num_promotions-1], val);
          else
            argp_error (state, "`%s' is a bad value for a promotion position", arg);
        }
      else if (next_arg_is_promote_duration)
        {
          next_arg_is_promote_duration = 0;
          double val = strtod (arg, &end);
          if (val > 0)
            promote_set_duration (xges_create->promotions[xges_create->num_promotions-1], val);
          else
            argp_error (state, "`%s' is a bad value for a promotion duration", arg);
        }
      else if (next_arg_is_file)
        {
          next_arg_is_file = 0;
          if (access (arg, R_OK) == 0)
            clip_set_file (xges_create->clips[xges_create->num_clips-1], arg);
          else
            argp_error (state, "could not open `%s' for reading. (%m)", arg);
        }
      else
        argp_error (state, "`%s' is not a recognized command.  did you mean `" CLIP_CMD "' or `" LAYER_CMD "' or '" PROMOTE_CMD "'?", arg);
      break;
    case ARGP_KEY_INIT:
      remember_commandline (xges_create, state->argv);
      state->child_inputs[0] = xges_create;  //clip
      state->child_inputs[1] = xges_create; //layer
      state->child_inputs[2] = xges_create; //promote
      break;
    case ARGP_KEY_FINI:
      if (next_arg_is_file)
        argp_error (state, "every `" CLIP_CMD "' needs a file.");
      if (xges_create->num_clips == 0 && xges_create->list_transitions == 0)
        argp_error (state, "you must specify a `" CLIP_CMD "'");
      if (next_arg_is_promote_duration ||
          next_arg_is_promote_position ||
          next_arg_is_promote_layer)
        argp_error (state, "incomplete `" PROMOTE_CMD "'");
      if (xges_create->num_clips)
        {
          clip_validate (xges_create->clips[xges_create->num_clips-1], state);
          if (clip_has_transition (xges_create->clips[0]))
            argp_error (state, "the first clip has nothing to transition from");
          if (xges_create->preview && xges_create->render_uri)
            argp_error (state, "cannot use --preview and --render simultaneously");
        }
      break;
    }
  return 0;
}

struct argp 
xges_create_argp = 
{ 
  xges_create_options, 
  parse_xges_create_opt, 
  LAYER_CMD " [OPTION...] " CLIP_CMD " FILE [OPTION...] " PROMOTE_CMD " LAYER SECS DURATION [OPTION...] ...", 
  "Generate a .xges file that has one or more clips on one or more layers.\v"
    "The default for --position is the length of the current layer.",
  child_parsers 
};

static void
list_transitions (FILE *fp)
{
  char **t;
  char **d;
  t = transitions;
  d = transition_desc;
  while (*t != 0)
    {
      fprintf (fp, "%s: %s\n", *t, *d);
      t++;
      d++;
    }
  return;
}

int
xges_create_generate (xges_create_t xges_create)
{
  int retval = 0;
  struct xges_create_details_t *t = (struct xges_create_details_t *) xges_create;
  if (t->list_transitions)
    list_transitions (stdout);
  else if (t->render_uri || t->preview)
    {
      int remove_file = 0;
      if (t->output_filename == NULL)
        {
          t->output_filename = "." PACKAGE ".xges";
          remove_file = 1;
        }
      retval = generate_xges (t);
      if (retval == 0)
        {
          char *cmd = NULL;
          if (t->render_uri)
            asprintf (&cmd, "ges-launch-1.0 --load=%s --outputuri=%s", 
                      t->output_filename, t->render_uri);
          else if (t->preview)
            asprintf (&cmd, "ges-launch-1.0 --load=%s", 
                      t->output_filename);
          printf("%s\n", cmd);
          retval = system (cmd);
          free (cmd);
          if (remove_file)
            remove (t->output_filename);
        }
    }
  else
    retval = generate_xges (t);
  return retval;
}
